#    most of this task is now taken from the standard application

set VERSION "CAEN"

#    put some default environment variables before loading common 

set env(RDOUThdwre) VX718

set LinuxDirectory  /MIDAS/Data_Acq/bin/Linux/
set DAProgram CAENExecV1

loadsource  sys.common

set frametitle "MIDAS Data Acquisition Experiment Control"

set VMEPresent 1
set VMEEnable 1

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_Data"]

loadsource sys.2D    ;# for 2D histogrammming feature 

set Supported_Modules [list V785 V775 V1190 V1290 V767]


proc Dump_Raw_Data {} {
    global Options

    set Options(Include.S9418) 0
    set Options(Include.VF48) 0
    set Options(Include.VME) 0

    Dump_Raw_VME_Data
}

proc create_spectra {} {
   global SpectrumServer HistMode EGreport Options
   global Supported_Modules

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1 &&  $Options(Histogram.$module) == 1} then {
           create_[set module]_spectra
       }
   }

   global 2Dlist 2Dpresent
   if {[llength $2Dlist] == 0} {
       if {$2Dpresent == 0} {2D_Histogram_Configuration; close-frame "[self frame-name].2dhistograms"}
   }
   Create_2D_Histograms

   set EGreport $EGreport_SAVE

   clear-footer
}

proc define_VME_addresses {} {
   global Supported_Modules Options

#    load hardware configuration into readout software

     ClearConfig

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1} then {define_[set module]_config}
   }

#    then the acquisition control module

     define_sac_config

}

proc click-vme {action args} {
    global VMEPresent VMEEnable

  if {$VMEPresent == 0} then {return}
  if {$VMEEnable == 0} then {:modify VME :value -1; return}

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }

   set-footer "sending $p to VME control task"
   set-busy
   if {[llength $args] == 0} then {
       set z [catch {send vmecontrol $p} m]
   } else {
       set z [catch {send vmecontrol $p [first $args]} m]
   }
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   if {$action == 3} then {define_VME_addresses}

   do_V767_setup $action
   do_V1190_setup  $action
   do_V1290_setup $action

   redisplay-vme
   update_button_states
}

proc redisplay-vme {} {
    global VMEEnable vmestate EGallow

  if {$VMEEnable == 0} then {return}

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} then {set vmestate -1; :modify VME :value $vmestate; return}

    set EGallow 0x10004

    set vl [read-register sac#1.Enable]
    if {$vl == 0x10004} then {
       midas-warning "Required hardware register is not known\n\
         Check your hardware configuration. Possibly a SETUP is needed"
       set vmestate -1
    } else {
       set vmestate [expr $vl & 1]
    }

    :modify VME :value $vmestate

    set EGallow 0xffffffff
}

proc Stop_Electronics {} {

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} {return}     ;#  no SAC is present

    write-registers sac#1.STR 0 
}

proc Go_Electronics {} {

    write-register NetVar.DACQ.PollTicks 50

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} {return}     ;#  no SAC is possible

    write-registers sac#1.Mode 1   ;#   CAEN mode

    write-registers sac#1.MON0 2
    write-registers sac#1.STR 9
    write-registers sac#1.ARM 0
}

proc Reset_Electronics {} {

    set sacpres [inform vmecontrol inquire saccount]
    if {[string length $sacpres] > 2} {set sacpres 0}

    if {$sacpres == 0} {return}     ;#  no SAC is  present

    write-registers sac#1.EVCLR 0
}

proc make-registers {} {
    global registerlist RegisterServer RDOUTserverList
    global EGreport EGallow

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

    set EGreport_SAVE $EGreport
    set EGreport -1

    set EGallow 0x10004

    set RegisterServer [first $RDOUTserverList]

    set rc [catch {read-register NetVar.EXEC.ID} m]
    if {$m == 0x10004} {
        foreach item $registerlist {
            set-footer "defining register [first $item]"
            define-register [second $item] [first $item] "[third $item]"
        }
    }

    clear-footer
    set EGreport $EGreport_SAVE
    set EGallow -1
}

proc make-executiveregisters {} {
    global executiveregisters RegisterServer RDOUTserverList
    global EGreport EGallow

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

    set EGreport_SAVE $EGreport
    set EGreport -1

    set EGallow 0x10004

    set RegisterServer [first $RDOUTserverList]

    set rc [catch {read-register NetVar.Executive.ID} m]
    if {$m == 0x10004} {
        foreach item $executiveregisters {
            set-footer "defining register [first $item]"
            define-register [second $item] [first $item] "[third $item]"
        }
    }

    clear-footer
    set EGreport $EGreport_SAVE
    set EGallow -1
}
