
#   package-load.tcl   -  core

proc init-packages_8.3 {} {
    global env        

    puts stdout "Setting up access for additional packages"

    if {$env(os) == "Windows NT"} then {set os Win} else {set os $env(os)}

#    Incr Tcl packges

    set env(ITCL_LIBRARY) [file join $env(MIDASBASE) packages itcl3.2]
    package ifneeded Itcl 3.2 "load itcl32[info sharedlibextension] Itcl"

    set env(ITK_LIBRARY) [file join  $env(MIDASBASE) packages itk3.2]
    package ifneeded Itk 3.2 "load itk32[info sharedlibextension] Itk"

#    Incr Widgets package

    set dir [file join $env(MIDASBASE) packages iwidgets4.0.2]
    source [file join $dir pkgIndex.tcl]

#    Tree package

    set env(TREE_LIBRARY) [file join  $env(MIDASBASE) packages Tree8.0.3]

    if {$os == "SunOS"} {puts stderr "package Tree not available for SunOS"; return}

    switch $env(platform)  {
       windows {package ifneeded Tree 8.0.3 "load Tree803.dll Tree"}
       unix    {package ifneeded Tree 8.0.3 "load libTkTree.so Tree"}
       default {puts stderr "package Tree not available for platform $env(platform)"}
    }
    
}

proc init-packages_8.4 {} {
    global env tcl_platform

#    puts stdout "Setting up access for additional packages"
    
#    Incr Tcl packges

    if {$tcl_platform(os) == "Windows NT"} then {set os Win} else {set os $tcl_platform(os)}

    set env(ITCL_LIBRARY) [file join $env(MIDASBASE) packages $os lib itcl3.3]
    load-package itcl3.3

    set env(ITK_LIBRARY) [file join $env(MIDASBASE) packages $os lib itk3.3]
    load-package itk3.3
    
#    packages which are OS dependent
    
    switch $os {
        Win    {
            load-package dde1.2
            load-package reg1.1
            load-package TclScript
            load-package tcom
            load-package Banking
            load-package treectrl1.1
        }        
        SunOS  {
            load-package expect5.42
        }
        Linux  {
            load-package expect5.42
        }
    }
    
#    packages which are available for all OSes

      load-package bwidget1.7
      load-package Img1.3
      load-package iwidgets4.0.2
      load-package Memchan2.2.1
      load-package Mk4tcl
      load-package snack2.2
      load-package tbcload1.4
#      load-package Tcldom2.6              ;# now in Common
      load-package tcldomxml2.6
      load-package Tclexpat2.6
#      load-package tcllib1.7              ;# now in Common
#      load-package tclsoap1.6.7           ;# now in Common
#      load-package tclx8.4                ;# now in Common
#      load-package Tclxml2.6              ;# now in Common
      load-package Tclxslt2.6
      load-package tdom0.8.0
      load-package tile0.5
      load-package Tkhtml2.0
      load-package tklib0.2
      load-package Tktable2.9
      load-package Trf2.1
      load-package vfs1.3
      load-package vu2.2
      load-package xotcl1.3.4

#    Tree package

    switch $os {
        Win    {
#           puts stderr "package Tree not available for Windows"
        }        
        SunOS  {
#           puts stderr "package Tree not available for SunOS"
        }
        Linux  {
           if {$tcl_platform(machine) != "ppc"} {
               set env(TREE_LIBRARY) [file join  $env(MIDASBASE) packages Tree8.0.3]
               package ifneeded Tree 8.0.3 "load libTkTree.so Tree"
           }
        }
    }
}

proc load-package {package} {
    global env tcl_platform auto_path ::auto_path

    if {$tcl_platform(os) == "Windows NT"} then {set os Win} else {set os $tcl_platform(os)}

    set dir [file join $env(MIDASBASE) packages $os lib $package]
    if {[file isdir $dir]} then {

        if {[lsearch -exact $::auto_path $dir] == -1} {lappend ::auto_path $dir}
        if {[lsearch -exact $auto_path $dir] == -1} {lappend auto_path $dir}

        source [file join $dir pkgIndex.tcl]
#       puts stdout "package $package available"
    }
}


proc init-packages {} {
    global env

     set version [info tclversion]
     
     switch $version {
        8.3   {init-packages_8.3}
        8.4   {init-packages_8.4}
        default {puts stderr "no additional packages supported for Tcl version $version"}
     }
}
