# caenps.tcl---CAEN HV PS control frame


set frame CAEN
set frametitle "CAEN HV PS Setup & Monitor"
set icontitle "CAEN"

set started 0

if {[info exists env(CAENetserver)]} then {set resource $env(CAENetserver)} \
    else {set resource CAEN}

set resource VME

set mname CAENet.V288
set V288Address  0xf00000

set savedirectory CAENPSSetup

set amplifier 1
set channel 0
set allamps 0
set allchans 0

set chstatusbits {OFF/ON OVC OVV UNV TRIP RUP RDW MAXV POL VSEL ISEL KILL HVEN NIM/TTL OUTCAL ALARM}

set selectampmenu {1 2 3 4}
set selectchanmenu   {0 1 2 3}
set amplist $selectampmenu
set chanlist $selectchanmenu

set Status ""



#    this is to allow the command just issued to be executed by the amp module
proc short_delay {} {after 10}
#    a longer delay needed when setting all channels
proc long_delay {} {after 100}


proc define-CAENetV288-registers {} {
   global resource mname

   set register "/$resource/$mname.*"
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"
   clear-footer

   define-register Address
   define-register Execute
   define-register Reset
   define-register Status
   define-register Data
}

proc define-register {name} {
   global resource mname

   set register "/$resource/$mname.$name"
   set attributes "-C $name"
   set-footer "defining register $register"
   EG "register :name '$register' :class 'xV288' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc write-register {name value} {
   global resource mname
   set register "/$resource/$mname.$name"
#   set-footer "writing $value to $register"
   EG "register :name '$register' :value '$value'"
#   clear-footer
}

proc read-register {name} {
   global resource mname
   set register "/$resource/$mname.$name"
#   set-footer "reading $register"
   set value [EG "register :name '$register' ?value"]
#   clear-footer
   return $value
}

proc click-amplifier v {
   global amplifier
   set amplifier $v
   redisplay
   read_ident
}

proc click-channel v {
   global channel
   set channel $v
   redisplay
}

proc click-select v {
    global setting
    click-channel $v
    set_param fg $setting(0)
    redisplay
}

proc repaint {} {
   global allamps allchans
   global channel amplifier
  :modify AMP-NUM    :value $amplifier
  :modify ALL-AMPS   :value $allamps
  :modify CHAN-NUM   :value $channel
  :modify ALL-CHANs  :value $allchans
}

proc click-save {w} {
   global env Status savedirectory
   global amplist chanlist amplifier
   global setting

   catch {file delete [file join $env(EXPERIMENTDIR) $savedirectory]}
   if {![file exists [file join $env(EXPERIMENTDIR) $savedirectory]]} then {
      set z [catch {:file mkdir [file join $env(EXPERIMENTDIR) $savedirectory]} m]
      if {$z != 0} then {midas-warning "Cannot create directory [file join $env(EXPERIMENTDIR) $savedirectory]\n $m"}
   }

   set currentamp $amplifier

   if {$w == "ALL"} then {
      set amps $amplist
   } else {
      set amps $amplifier
   }

   set-busy
   set contents ""
   set r 0
   foreach amplifier $amps {
         set Status {}
         set-footer "Saving settings for amplifier $amplifier..."
         append contents "Settings saved for amplifier $amplifier"

#    obtain values for this amplifier
 
         foreach chan $chanlist {
             set r [read_ch $chan]
             if {$r != 0} then {break}
             lappend Status [list $setting(0) $setting(1) $setting(2) $setting(3) $setting(4) $setting(5)]
         }
         if {$r != 0} then {break}
         lappend Status $setting(6)

#    save amplifier status to file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier] w]
         puts $f $Status
         close $f
         append contents " to file [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier]\n"
   }
   clear-busy

   set amplifier $currentamp

   if {$r != 0} then {
      midas-warning "Save of amplifier settings to $env(EXPERIMENTDIR)/$savedirectory abandoned"
      clear-footer
      redisplay
      return
   }

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been saved!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings saved to $env(EXPERIMENTDIR)/$savedirectory"

   redisplay
}

proc click-restore {w} {
   global env Status savedirectory
   global amplist chanlist amplifier channel allchans
   global params

   set currentamp $amplifier
   set currentchan $channel
   set allchans 0

   if {$w == "ALL"} then {
      set amps $amplist
   } else {
      set amps $amplifier
   }

   set-busy
   set contents ""
   set r 0
   foreach amplifier $amps {
         set-footer "Restoring settings for amplifier $amplifier..."
         append contents "Settings restored for amplifier $amplifier"

#    read amplifier settings from file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier] r]
         set Status [gets $f]
         close $f
         append contents " from file [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier]\n"

#    restore values for this amplifier

         set c 0
         foreach channel $chanlist {
             set settings [lindex $Status $c]
             incr c
             set p 0
             foreach param $params {
                 set r [set_param $param [lindex $settings $p]]
                 incr p
                 if {$r != 0} then {break}
             }
             if {$r != 0} then {break}
         }
         if {$r != 0} then {break}

         set r [set_offset [last $Status]]
         if {$r != 0} then {break}
   }
   clear-busy

   set amplifier $currentamp
   set channel $currentchan

   if {$r != 0} then {
      midas-warning "Restore of amplifier settings from $env(EXPERIMENTDIR)/$savedirectory abandoned"
      clear-footer
      redisplay
      return
   }

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been restored!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings restored from $env(EXPERIMENTDIR)/$savedirectory"

   redisplay
}


proc do-update {} {
   global channel
   read_ps
   read_ch $channel
}

proc redisplay {} {repaint; do-update}

proc click arg {
   global debug allamps allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      AMP-SEL     -
      AMP-NUM     {click-amplifier $v}
      ALL-AMPS    {set allamps $v}

      CHAN-SEL      -
      CHAN-NUM      {click-channel $v}
      ALL-CHANS     {set allchans $v}

      PROBE        {probe_for_amplifiers}

      V0         {set_param v0  $v;long_delay; redisplay}
      V1         {set_param v1  $v;long_delay; redisplay}
      I0         {set_param i0  $v;long_delay; redisplay}
      I1         {set_param i1  $v;long_delay; redisplay}
      TRIP       {set_param tp  $v;long_delay; redisplay}
      UP         {set_param up  $v;long_delay; redisplay}
      DN         {set_param dn  $v;long_delay; redisplay}

      ENABLE     {click-enable $v}

      SAVERESTORE   {click-saverestore $v}
      OTHER         {click-other $v}

      REDISPLAY    {redisplay}
      default      {puts stdout $arg}
   }
}


proc make-frame {} {
  global env
  global selectampmenu selectchanmenu
  global chstatusbits

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number AMP-NUM  :label "Power Supply number:" :min 1 :max 24 :w 5  :valuex 20c
      :menubutton AMP-SEL :label "select:" :strings $selectampmenu
      :hskip 10
      :choice ALL-AMPS :label "Act on PS?" :valuex 18c :strings [list Current All] :value 0
      :next-row T
      :text STAT0 :label "Status:  0"  :value 0x0000 :w 8 :valuex 10c
      :text STAT1 :label 1  :value 0x0000 :w 8 :valuex 5c
      :text STAT2 :label 2  :value 0x0000 :w 8 :valuex 5c
      :text STAT3 :label 3  :value 0x0000 :w 8 :valuex 5c
      :next-row T
      :text VMON0 :label "Vmon:  0"  :value 9999 :w 8 :valuex 10c
      :text VMON1 :label 1  :value 9999 :w 8 :valuex 5c
      :text VMON2 :label 2  :value 9999 :w 8 :valuex 5c
      :text VMON3 :label 3  :value 9999 :w 8 :valuex 5c
      :next-row T
      :text IMON0 :label "Imon:  0"  :value 9999 :w 8 :valuex 10c
      :text IMON1 :label 1  :value 9999 :w 8 :valuex 5c
      :text IMON2 :label 2  :value 9999 :w 8 :valuex 5c
      :text IMON3 :label 3  :value 9999 :w 8 :valuex 5c
      :next-row T
      :text VMAX0 :label "Vmax:  0"  :value 9999 :w 8 :valuex 10c
      :text VMAX1 :label 1  :value 9999 :w 8 :valuex 5c
      :text VMAX2 :label 2  :value 9999 :w 8 :valuex 5c
      :text VMAX3 :label 3  :value 9999 :w 8 :valuex 5c
      :next-row T

      :number CHAN-NUM  :label "Channel number:" :min 0 :max 3 :w 5 :valuex 20c
      :menubutton CHAN-SEL :label "select:" :strings $selectchanmenu
      :hskip 10
      :choice ALL-CHANS :label "Act on Channels?" :valuex 18c :strings [list Current All] :value 0

    :panel P2 :layout R :padx 10
      :choice STATUS  :label "CH Status" :exclusive F :strings $chstatusbits :w 10 :layout V :deaf T

    :panel P1 :layout L :padx 10
      :layout V; :col-gap 15; :row-gap 15

      :checkbox ENABLE :label "Channel enabled" :valuex 20c

      :slider V0     :label "V0 (Volt)"       :min 0 :max 8000  :valuex 20c
      :slider I0     :label "I0 (microamp)"   :min 0 :max 3000  :valuex 20c
      :slider V1     :label "V1 (Volt)"       :min 0 :max 8000  :valuex 20c
      :slider I1     :label "I1 (microamp)"   :min 0 :max 3000  :valuex 20c

      :slider TRIP   :label "Trip (1/100sec)"   :min 1 :max 9999  :valuex 20c
      :slider UP     :label "RampUp (V/sec)"    :min 1 :max 500  :valuex 20c
      :slider DN     :label "RampDown (V/sec)"  :min 1 :max 500  :valuex 20c

    :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :col-gap 10
      :button REDISPLAY

      :menubutton SAVERESTORE :label "Save/Restore"  \
         :strings [list "Save All Power Supplies" "Save Current Power Supply" "Restore All Power Supplies" "Restore Current Power Supply"]
      :menubutton OTHER :label "Other actions" \
         :strings [list  "Kill all channels" "Clear alarm output" "Enable front panel" "Disable front panel" "Select TTL level" "Select NIM level"]

      :button PROBE :label "Probe for Power Supplies"
#      :button HELP  :command click-help

      :show T

}

proc click-saverestore {v} {set-footer "$v"}



proc read_ps {} {
   global widgets setting
   global amplifier 
   global response

    set OpCode 1

#    write command  to the Tx buffer

    write-register Data "$amplifier $OpCode"

#    execute the CAENet operation

    short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        if {[expr $status & 255] == 255} then {
            midas-warning "The power supply is not responding\nCheck that power supply $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return 1
    }

#    read the received data

    set response [read-register Data]

      :modify VMON0 :value [expr [lindex $response 0]]
      :modify IMON0 :value [expr [lindex $response 1]]
      :modify VMAX0 :value [expr [lindex $response 2]]
      :modify STAT0 :value [lindex $response 3]

      :modify VMON1 :value [expr [lindex $response 4]]
      :modify IMON1 :value [expr [lindex $response 5]]
      :modify VMAX1 :value [expr [lindex $response 6]]
      :modify STAT1 :value [lindex $response 7]

      :modify VMON2 :value [expr [lindex $response 8]]
      :modify IMON2 :value [expr [lindex $response 9]]
      :modify VMAX2 :value [expr [lindex $response 10]]
      :modify STAT2 :value [lindex $response 11]

      :modify VMON3 :value [expr [lindex $response 12]]
      :modify IMON3 :value [expr [lindex $response 13]]
      :modify VMAX3 :value [expr [lindex $response 14]]
      :modify STAT3 :value [lindex $response 15]

   return 0
}

proc read_ch {n} {
   global amplifier 
   global response

    set OpCode [expr $n << 8 | 2]

#    write command  to the Tx buffer

    write-register Data "$amplifier $OpCode"

#    execute the CAENet operation

    short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        if {[expr $status & 255] == 255} then {
            midas-warning "The power supply is not responding\nCheck that power supply $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return 1
    }

#    read the received data

    set response [read-register Data]

      set status [lindex $response 0]

      :modify STATUS :value $status
      :modify ENABLE :value [expr $status & 1]

      :modify V0 :value [lindex $response 3]
      :modify I0 :value [lindex $response 4]
      :modify V1 :value [lindex $response 5]
      :modify I1 :value [lindex $response 6]

      :modify TRIP :value [lindex $response 7]
      :modify UP   :value [lindex $response 8]
      :modify DN   :value [lindex $response 9]

   return 0
}


proc read_ident {} {
   global V288Address
   global amplifier 
   global response

   read-register Reset

#    write OpCode 0  -  module identifier  -  to the Tx buffer

    write-register Data "$amplifier 0"

#    execute the CAENet operation

     short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        if {[expr $status & 255] == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return
    }

#    read the received data

    set response [read-register Data]

    set n418 ""
    for {set i 0} {$i < 16} {incr i} {
       append n418 [format %c [lindex $response $i]]
    }

     set-footer "The amplifier has answered: $n418"
}

proc probe_for_amplifiers {} {
   global selectampmenu amplist amplifier 

   set amplist {}
   foreach amp $selectampmenu {

       set-footer "Probing for Power Supply $amp"

#    write OpCode 0  -  module identifier  -  to the Tx buffer

       write-register Data "$amp 0"

    short_delay

#    execute the CAENet operation

       set status [read-register Execute]
       if {$status == 0 || $status == 7} then {
           set response [read-register Data]   ;#    read the received data
           set model ""
           for {set i 0} {$i < 4} {incr i} {
               append model [format %c [lindex $response $i]]
           }
           if {$model == "N470"} then {lappend amplist $amp}
       }
   }
   
   clear-footer

   :modify AMP-SEL :strings $amplist
    
   if {[llength $amplist] == 0} then {
       midas-warning "No Power Supplies where detected\nCheck connections and NIM bin power"
   } else {
       set amplifier [first $amplist]
   }
}

proc click-enable {v} {

   switch $v {
       0  {set_param off 0}
       1  {set_param on 0}
   }

   long_delay; redisplay
}

proc click-other {action} {

    switch $action {
        "Kill all channels"   {set_param kill 0}
        "Clear alarm output"  {set_param clr 0}
        "Enable front panel"  {set_param en 0}
        "Disable front panel" {set_param dis 0}
        "Select TTL level"    {set_param ttl 0}
        "Select NIM level"    {set_param nim 0}
    }
}
        
proc set_param {p v} {  
   global channel allchans allamps
   global amplist chanlist amplifier 
   global response

#    note ch is channel number (0-3) or all channels (16)

    insert-debug "set_param $p $v"

  switch $p {
     v0     {set opcode 3}
     v1     {set opcode 5}
     i0     {set opcode 4}
     i1     {set opcode 6}
     tp     {set opcode 7}
     up     {set opcode 8}
     dn     {set opcode 9}
     on     {set opcode 10}
     off    {set opcode 11}
     kill   {set opcode 12}
     clr    {set opcode 13}
     en     {set opcode 14}
     dis    {set opcode 15}
     ttl    {set opcode 16}
     nim    {set opcode 17}
     default   {return}
  }                
 

   if {$allchans == 0} then {set chs $channel} else {set chs $chanlist}

   if {$allamps == 0} then {set amps $amplifier} else {set amps $amplist}
   set currentamp $amplifier

   foreach amplifier $amps {
     foreach ch $chs {
 
        if {$opcode <= 11} then {
            set OpCode [expr $ch << 8 | $opcode]
        } else {
            set OpCode $opcode
        }

#    write command  to the Tx buffer
       
       if {$opcode <= 9} then {
           write-register Data "$amplifier $OpCode $v"
       } else {
           write-register Data "$amplifier $OpCode"
       }

       short_delay

#    execute the CAENet operation

       set status [read-register Execute]
       if {$status != 0} then {
          if {[expr $status & 255] == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
          } else {
            midas-warning "CAENet: Error number [format 0x%x $status] received"
          }
          set amplifier $currentamp
          redisplay
          return 1
       }
   } }

   set amplifier $currentamp
   return 0
}


proc init {} {
   global EGreport V288Address

   set EGreport -1

   define-CAENetV288-registers

   write-register Address $V288Address
}

proc start args {
   global started

   if {$started == 0} then {make-frame; set started 1} else {open-frame}

   init
   if {$started < 2} then {probe_for_amplifiers; set started 2}
   read_ident

   redisplay
}

